#ifndef CAMERA_COMMANDS_H
#define CAMERA_COMMANDS_H

#include "Shell.h"
#include "ICameraProvider.h"

#include <P1CameraCamera.hpp>

#include <string>
#include <vector>
#include <memory>
#include <thread>
#include <map>

class CameraCommands : public ICameraProvider
{
private:
    std::map<int,std::unique_ptr<std::thread>> mImageThreads; // must be destructed after mCamera
    std::map<int,std::unique_ptr<std::thread>> mLiveViewThreads; // must be destructed after mCamera
    std::map<int, P1::CameraSdk::Camera> mCameras;

    bool saveImages = false;

    static std::atomic<int> recvCount;

    void PrintPropertyValue(const P1::CameraSdk::PropertyValue& value, const std::string name, const int id) const;

    virtual P1::CameraSdk::Camera& GetCameraConnected() override
    {
        static P1::CameraSdk::Camera dummy;
        if (mCameras.size() == 0) return dummy;

        return mCameras.begin()->second;
    };

    virtual std::map<int,P1::CameraSdk::Camera>& AllCameras() override
    {
        return mCameras;
    }

    virtual P1::CameraSdk::Camera* GetCamera(int id) override
    {
        if (mCameras.find(id) == mCameras.end()) return nullptr;
        else
        {
            return &mCameras[id];
        }
    }

    bool mImageThreadRunnable = false;
    void ImageThreadMain(P1::CameraSdk::Camera& camera);
    void TerminateImageThread();

    bool mLiveViewThreadRunnable = false;
    void LiveViewThreadMain(P1::CameraSdk::Camera& camera);
    void TerminateLiveViewThread();

public:
    CameraCommands(Shell& shell);
    ~CameraCommands();

    void PrintList();
    void Open(std::vector<std::string>& tokens);
    void Close(std::vector<std::string>& tokens);
    void Capture(std::vector<std::string>& tokens);
    void EnableReceiving(std::vector<std::string>& tokens);
    void SetLiveView(std::vector<std::string>& tokens);
    void SetSave(std::vector<std::string>& tokens);
    void SetHostCapacity(std::vector<std::string>& tokens);
    void Pending(std::vector<std::string>& tokens);

    void List(std::vector<std::string>& tokens);
};

#endif // CAMERA_COMMANDS_H