# CamTool

**Command line tool that demonstrates the features and API's of CameraSDK. The source is demonstrating the usage of CameraSDK, using a command line parser infrastructure.**

**We recommend you use this application to explore how to control your Phase One camera through our SDK.**

## Build Requirements

**Linux**:

* Ubuntu 20.04 (Focal) (*ARM64* or *x86_64*)
* GCC 8.3 or later
* CMake 3.13 or later

We have included a `Dockerfile`, that provides the required environment.

**Windows**:

* Windows 10 (_x64_)
* MSVC 2019 or later
* CMake 3.13 or later

## Build

The project is compiled using CMake:

```sh
$ mkdir build && cd build
$ cmake ..
$ cmake --build .
```

This will trigger the CMake build system to automatically download our SDK libraries for your system.

### Using the Dockerfile

If you wish to use Docker to build the samples, follow these steps. (The *CWD* is assumed to be repository root dir.)

One of the great things about Docker, is that it includes QEMU to run containers targeted for another CPU. This enables you to run *ARM64* code on an *x86_64* host - or vice versa. 

The example below builds the Docker image for ARM64 (aka. `aarch64`), regardless of the host system's architecture:

```sh
$ docker build --platform linux/arm64 -t <IMAGE_NAME> .
$ mkdir build
$ docker run -ti --rm -v `pwd`:/app -u `id -u`:`id -g` --platform linux/arm64 <IMAGE_NAME> bash -c "cd /app/build && cmake .. && cmake --build ."
```

This will use the Docker container to compile all targets. Replace the word `<IMAGE_NAME>` with a name for your Docker image - like `p1-sdk-builder`.

If you are running on an ARM based host, and would like to run a *x86_64* container, just replace the value of the `--platform` argument with `linux/amd64`. This will build the image for the *x86_64* architecture.

## File overview

* `Shell.h/cpp`: A crude interactive shell for entering commands
* `*Commands.h/cpp`: The implementation of camera related commands that are triggered from the shell. SDK is called from here
* `main.cpp`: The entry point that initiates the *Shell* and setup the available *Commands*

## Available commands

This is a print of the `help` command's output:

    Available commands:
      capture      Trigger a capture on the open camera
      close        Close the currently opened camera
      exit         Quit the application
      help         Print this help text
      hostcapacity <capacity in MB> Tells the camera how much capacity this host has available for images
      list         Prints a list of visible camera (USB only)
      liveview     [on|off] Shows config and Enable/Disable receiving live view images
      notif        Enable/Disable notifications
      open         [IP=X.X.X.X|serialno=<serialno>] [password=<password>] [retries=<num>] [timeout=<secs>] Open an IP or USB camera
      opened       List all opened cameras
      pending      List pending images count, for all cameras
      property     Control of properties
      quit         Quit the application
      recv         <on|off> Enable / Disable receiving captured images.
      save         <on|off> Save received images to CWD

*Note that the `list` command can only see cameras that is connected via USB.*

## Open connection to a camera

Run the ``CamTool`` executable and use the ``open`` command, to open USB connected camera by default. 

    CamTool> open

This will open the first found USB connected camera. If your camera is connected via TCP/IP, you need to specify its IP address as an argument:

    CamTool> open IP=1.2.3.4

## List camera properties

With a camera already opened, run the following command:

    CamTool> property list

THis will show a long list of every exposed camera properties. For each line, 4 parameters are shown: *Id*, *Name*, *(Type)*, *Value*.

### Changing a property

Let us try to change the aperture (Id: *1003*) on the camera. First we show the current property value:

    CamTool> property get 1003
    PropertySpecification:
      ID:               1003
      Name/String:      Aperture
      Value:            (Float) : 2.8
      Group:            {10}IQ\{2}Capture Setup
      Read Only:        True
      Description:
      Enabled:          True
      User Level:       5
      Select Only:      True
      DisplayAsHex:     True
      Refresh Required: False
      Unit:             f/
      Display Index:    0
      Default:          (Float) : 2.8
      Range:            (2.8, 22)
      ValueList:        19 entries { 2.8, 3.2, 3.5, 4.0, 4.5, 5.0, 5.6, 6.3, 7.1, 8.0, 9.0, 10, 11, 12, 14, 16, 18, 20, 22 }

The possible values the property can take is shown at the last line of the output: *ValueList*. The current value is *2.8*, so let's set it to *18*.

    CamTool> property set 1003 18
    Exception caught while setting property: "Property was not set because it might not exist, is read only, disabled, etc.", ErrorCode: -25

Okay, that failed. This is likely because the camera is set to automatic exposure control. We need to change the *mode* to *Manual*:

    CamTool> property set 1000 0

Now we can control the aperture manually:

    CamTool> property set 1003 18
    CamTool> property get 1003
    PropertySpecification:
      ID:               1003
      Name/String:      Aperture
      Value:            (Float) : 18
      Group:            {10}IQ\{2}Capture Setup
      Read Only:        False
      Description:
      Enabled:          True
      User Level:       5
      Select Only:      True
      DisplayAsHex:     True
      Refresh Required: False
      Unit:             f/
      Display Index:    0
      Default:          (Float) : 2.8
      Range:            (2.8, 22)
      ValueList:        19 entries { 2.8, 3.2, 3.5, 4.0, 4.5, 5.0, 5.6, 6.3, 7.1, 8.0, 9.0, 10, 11, 12, 14, 16, 18, 20, 22 }

## Capture a photo

Use the following commands:

    CamTool> recv on
    CamTool> save on
    CamTool> capture

The `recv` command enable transferring of images from the camera to the host computer. The `save` command enables writing these images to disk. (Otherwise they are just discarded.)

You should hear the shutter mechanism click in the camera. (If electronic shutter is not enabled.) The IIQ file is now saved in the applications CWD.