#ifndef SHELL_H
#define SHELL_H

#include <string>
#include <vector>
#include <functional>
#include <map>

class Shell
{
public:
    using HandlerFunction = std::function<void(std::vector<std::string>&)>;
    using SimpleHandlerFunction = std::function<void()>;

private:

    struct Handler
    {
        const std::string description;
        HandlerFunction argFunction;
        SimpleHandlerFunction simpleFunction;
    };

    std::map<const std::string,Handler> handlers;
    std::string prefix;
    std::string currentLine;
    bool shouldRun;

    void ThrowIfExists(const std::string command);

public:
    Shell(const std::string& prefix);

    void AddHandler(const std::string command, HandlerFunction handler, const std::string helpText = "");
    void AddHandler(const std::string command, SimpleHandlerFunction handler, const std::string helpText = "");

    void Run();

    void ExecLine(std::string line);

    void SignalStop();

    void HelpHandler();

    static std::string ToLower(const std::string s);
    static bool BeginsWith(const std::string s, const std::string beginsWith, bool ignoreCase);
    static int IndexOfArgument(const std::string argument, const std::vector<std::string> tokens, bool prefixIsMatch = false);
};

#endif // SHELL_H