#include <iostream>
#include <string>
#include <vector>
#include <fstream>

#include "Shell.h"
#include "CameraCommands.h"
#include "NotificationCommands.h"
#include "PropertyCommands.h"

#include <P1Camera.hpp>

// Make the windows console (on Win 10) handle UTF-8 strings,
// by allowing the app to set the code page	
#ifdef WIN32
#include <Windows.h>
#undef ERROR
#endif

using std::string;

void loggingHandler(const char* message, P1::CameraSdk::LogMessageLevel level)
{
	using P1::CameraSdk::LogMessageLevel;
	switch (level)
	{
	case LogMessageLevel::ERROR:
		std::cout << "\u001b[31mSDK: " << message << "\u001b[0m"; // RED
		break;
	case LogMessageLevel::WARNING:
		std::cout << "\u001b[33mSDK: " << message << "\u001b[0m"; // YELLOW
		break;
	case LogMessageLevel::INFO:
	case LogMessageLevel::VERBOSE:
	default:
		std::cout << "\u001b[90mSDK: " << message << "\u001b[0m"; // CYAN
		break;
	}
}

// ---------------------------------------------------------------------------
// --- main
// ---------------------------------------------------------------------------
int main(int argc, const char* argv[])
{

#ifdef WIN32
	// Enable UTF-8 console code page
	// Since all SDK string are UTF8, which is not handled correctly by default on Windows
	SetConsoleOutputCP(CP_UTF8);
#endif

	P1::CameraSdk::SetSdkLoggingHandler(loggingHandler);

	P1::CameraSdk::Version sdkVer = P1::CameraSdk::GetSdkVersion();

    std::cout << "CamTool: Test CameraSDK using command line interpreter\n(CameraSDK: "
		<< sdkVer.major << "." << sdkVer.minor << "." << sdkVer.build
		<< " : " << P1::CameraSdk::GetSdkBuildDate() << " ("
		<< P1::CameraSdk::GetSdkBuildHash()
		<< ")" << ", " << P1::CameraSdk::GetSdkBuildCompiler()
		<< ")\nUse command 'help' for a list of available commands.\n";

    Shell shell("CamTool>");

    CameraCommands camCmds(shell);
	NotificationCommands notifCmds(shell, camCmds);	// CameraCommands act as ICameraProvider
	PropertyCommands propCmds(shell, camCmds);	// CameraCommands act as ICameraProvider

    shell.AddHandler("help", [&shell](){ shell.HelpHandler(); }, "Print this help text");
    shell.AddHandler("exit", [&shell](){ shell.SignalStop();  }, "Quit the application");
    shell.AddHandler("quit", [&shell](){ shell.SignalStop();  }, "Quit the application");

    shell.Run();

    return 0;
}